import os


def in_ipynb():
    try:
        get_ipython()
        return True
    except NameError:
        return False


def create_if_not_exists(folder):
    if not os.path.exists(folder):
        os.mkdir(folder)



from typing import Any, List
import os
from .constants import INPUT_DIR


def input_from_file(file, input_folder=INPUT_DIR):
    with open(os.path.join(input_folder, file), "r") as f:
        lines = f.readlines()

    def input():
        line = lines.pop(0)
        return line
    return input


def default_select_from_list(list, title="Select an item:"):
    """
    Select an item from a list.
    """
    if len(list) == 0:
        raise ValueError("List is empty")
    if len(list) == 1:
        return list[0]
    for i, item in enumerate(list):
        print(f"{i} - {item}")
    try:
        choice = input(f"{title} ")
    except KeyboardInterrupt:
        print("\nExiting...")
        exit()
    try:
        return list[int(choice)]
    except (ValueError, IndexError):
        raise ValueError(f"Invalid choice: {choice}")


def select_in_list(choices: List[Any], title="Select an item:"):
    str_choices = list(map(str, choices))
    if len(str_choices) == 0:
        raise ValueError("List is empty")
    if len(str_choices) == 1:
        return str_choices[0]
    for i, item in enumerate(str_choices):
        print(f"{i} - {item}")
    try:
        choice = input(f"{title} ")
    except KeyboardInterrupt:
        print("\nExiting...")
        exit()
    try:
        return choices[int(choice)]
    except (ValueError, IndexError):
        raise ValueError(f"Invalid choice: {choice}")


def select_input_instance(input_folder=INPUT_DIR):
    return select_in_list(os.listdir(input_folder), title="Select an input instance:")



import os
import pickle
from collections import namedtuple
from typing import Any, Callable, Optional, List
from .constants import DEBUG_DIR
from .utils import create_if_not_exists


Pipeline = namedtuple("Pipeline", ["id", "run"])
Stats = namedtuple("Stats", ["pipeline_id", "times", "objectives"])


def prepare_pipeline(name: str, prep_instance: Callable[[], Any], prep_model: Callable[[Any], Any], run_model: Callable[[Any, Optional[int]], float]):
    """Prepare a pipeline.

    Args:
        prep_instance (Callable[[Any], Any]): Prepared the instance to the pipeline.
        prep_model (Callable[[Any], Any]): From instance to model.
        run_model (Callable[[Any, Optional[int]], float]): From model to solution.
    """
    def run(max_time=None):
        print("Preparing instance...")
        prepared_instance = prep_instance()
        print("Instance prepared.")
        print("Building model...")
        prepared_model = prep_model(prepared_instance)
        print("Model built.")
        print("Solving model...")
        obj = run_model(prepared_model, max_time)
        print("Model solved.")
        return obj
    return Pipeline(name, run)


def evaluate_pipelines(pipelines: List[Pipeline], pb_name="", debug_dir=DEBUG_DIR):
    """Evaluate pipelines with instance

    Args:
        instance (Any): The input instance given to your pipeline
        pipelines (List[Pipeline]): The pipelines
        pb_name (str, optional): The name of the problem. Defaults to "".
        debug_dir ([type], optional): The folder where the debug files are saved. Defaults to DEBUG_DIR.
    """
    create_if_not_exists(debug_dir)
    pipelines_stats = []
    for pipeline in pipelines:
        pipeline_path = os.path.join(debug_dir, f"{pb_name}.{pipeline.id}.pkl")
        if os.path.exists(pipeline_path):
            with open(pipeline_path, 'rb') as f:
                pipelines_stats.append(pickle.load(f))
        else:
            pipeline_id, run = pipeline
            pipeline_stats = Stats(pipeline_id, [], [])
            for max_t in [5, 15, 30, 60, 120]:
                print(f"Solving model {pipeline_id}(max_t={max_t})...")
                obj = run(max_time=max_t)
                pipeline_stats.times.append(max_t)
                pipeline_stats.objectives.append(obj)
                print(f"Model {pipeline_id}(max_t={max_t}) stoped. [{obj}]")
            with open(pipeline_path, 'wb') as f:
                print(pipeline_stats)
                pickle.dump(pipeline_stats, f)
            pipelines_stats.append(pipeline_stats)
    for pipeline_stats in pipelines_stats:
        print(f"Pipeline {pipeline_stats.pipeline_id}")
        for t, obj in zip(pipeline_stats.times, pipeline_stats.objectives):
            print(f"\t{t}s: {obj}")
        print()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="optiz",
    version="0.0.7",
    license="MIT",
    description="Optimization library for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Lucas Marandat",
    author_email="lucas.mrdt@gmail.com",
    url="https://github.com/lucasmrdt/optiz",
    keywords=["optimisation"],
    packages=setuptools.find_packages(),
    install_requires=[
        "ortools==9.2.9972"
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    py_modules=["optiz"],
)



import os
from typing import Any, Callable
from ortools.sat.python import cp_model

from ..constants import OUTPUT_DIR
from ..utils import create_if_not_exists


class CPSolSaver(cp_model.CpSolverSolutionCallback):
    def __init__(self, name: str, variables, sol_to_str: Callable[[Any], str], output_dir=OUTPUT_DIR):
        cp_model.CpSolverSolutionCallback.__init__(self)
        create_if_not_exists(output_dir)
        self.__variables = variables
        self.__solution_count = 0
        self._name = name
        self._output_dir = output_dir
        self._sol_to_str = sol_to_str

    def OnSolutionCallback(self):
        self.__solution_count += 1
        save_path = os.path.join(self._output_dir, f"{self._name}.txt")
        variables = [self.Value(v) for v in self.__variables]
        with open(save_path, "w") as f:
            f.write(self._sol_to_str(variables))

    def SolutionCount(self):
        return self.__solution_count
